# -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os.path

import blorb
import ifiction
import zcode
import glulx
import tads
import adrift
from ifformaterror import IFFormatError


class BabelError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


handlers = [zcode, glulx, tads, adrift]
extension_map = {}


for h in handlers:
    for ext in h.get_extensions():
        extension_map[ext] = h


def get_storydata(file_handle, filename=None):
    file_buffer = file_handle.read()
    result = None
    try:
        b = blorb.Blorb(file_buffer)
    except IFFormatError as e:
        extension = None
        if filename is not None:
            basename = os.path.basename(filename)
            extindex = basename.rfind('.')
            if extindex >= 0 and extindex < len(basename) - 1:
                extension = basename[extindex + 1:len(basename)]
                if extension is not None and extension in extension_map:
                    handler = extension_map[extension]
                    if handler.is_a(file_buffer):
                        result = handler.get_information(file_buffer)
            if result is None:
                for handler in handlers:
                    if handler.is_a(file_buffer):
                        result = handler.get_information(file_buffer)
    else:
        xml = b.get_metadata()
        result = ifiction.get_stories_from_ifiction(xml)[0]
    if result is not None:
        result.story_file = filename
    else:
        raise BabelError('Unknown story format.')
    return result


def get_storydata_from_filename(filename):
    with open(filename, 'rb') as f:
        return get_storydata(f, filename)


if __name__ == "__main__":
    name = sys.argv[1]
    with open(name) as f:
        s = get_storydata(f, name)
        if s != None:
            for attr in [attr for attr in dir(s) if not attr.startswith('_')]:
                print str(attr) + ": " + str(getattr(s, attr))
        else:
            print 'Not a valid story file ({0}).'.format(name)
