# -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


from struct import unpack_from

from ifformaterror import IFFormatError

class Blorb:
    def __init__(self, data):
        self.data = data
        if len(self.data) < 16 or not self.data.startswith('FORM') \
                or not self.data[8:12] == 'IFRS':
            raise IFFormatError('Not a BLORB.')

    def get_chunk(self, chunk_id):
        i = 12
        while i < len(self.data) - 8:
            length = unpack_from('>I', self.data, i + 4)[0]
            if self.data[i:i + 4] == chunk_id:
                return self.data[i + 8:i + 8 + length]
            if length % 2 != 0:
                length = length + 1
            i = i + length + 8
        return None

    def get_resource(self, resource, n):
        index = self.get_chunk('RIdx')
        number_resources = unpack_from('>I', index)[0]
        i = 0
        while (i < number_resources):
            if (index[4 + i * 12:8 + i * 12] == resource and
                    unpack_from('>I', index, 8 + i * 12)[0] == n):
                rsc_start = unpack_from('>I', index, 12 + i * 12)[0]
                rsc_length = unpack_from('>I', self.data, rsc_start + 4)[0]
                return self.data[rsc_start + 8:rsc_length]
            i = i + 1
        return None

    def get_resources(self):
        index = self.get_chunk('RIdx')
        number_resources = unpack_from('>I', index)[0]
        result = []
        for i in xrange(0, number_resources):
            rsc_name = index[4 + i * 12:8 + i * 12]
            rsc_number = unpack_from('>I', index, 8 + i * 12)[0]
            result.append((rsc_name, rsc_number))
        return result

    def get_metadata(self):
        return self.get_chunk('IFmd')

if __name__ == "__main__":
    import sys
    name = sys.argv[1]
    with open(name) as f:
        b = Blorb(f.read())
        print b.get_metadata()
        print b.get_chunk('IFmd')
