# -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


from binascii import hexlify
import re
import struct

from storydata import StoryData


INFORM_OFFSET = 36
MMAP_SIZE_OFFSET = 12
SERIAL_OFFSET = 54
CHECKSUM_OFFSET = 32
INFORM_IDENTIFIER = 'Info'
SERIAL_LENGTH = 6


def get_extensions():
    return ['ulx']


def is_a(file_buffer):
    return len(file_buffer) >= 256 and file_buffer.startswith('Glul')


def get_information(file_buffer):
    s = StoryData()
    s.ifid = calc_ifid(file_buffer)
    s.format = 'glulx'
    return s


def calc_ifid(file_buffer):
    ifid_match = re.search('UUID://([^/]+)//')
    if ifid_match is not None:
        return ifid_match.group(1)
    else:
        # What is INFORM_LOCATION???????
        is_inform = file_buffer[INFORM_LOCATION:INFORM_OFFSET +
                                len(INFORM_IDENTIFIER)] == INFORM_IDENTIFIER
        mmap_size = hexlify(struct.unpack('>I', file_buffer,
                                          MMAP_SIZE_OFFSET)[0]).upper()
        release_number = str(struct.unpack('>H', file_buffer,
                                           RELEASE_NUMBER_OFFSET)[0])
        serial_number = re.sub('[^[[:alnum:]]]', '-',
                               file_buffer[SERIAL_OFFSET:SERIAL_OFFSET +
                                           SERIAL_LENGTH])
        checksum = hexlify(struct.unpack('>I', file_buffer,
                                         CHECKSUM_OFFSET)[0]).upper()
        ifid = 'GLULX-'
        if not is_inform:
            ifid = ''.join([ifid, mmap_size])
        else:
            ifid = ''.join([ifid, release_number, '-', serial_number])
        ifid = ''.join([ifid, '-', checksum])
        return ifid
