# -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.

import os.path
from glob import glob


class StoryData(object):
    def __init__(self, data=None):
        if data is None:
            self.story_file = None
            self.ifid = None
            self.bafn = None
            self.format = None
            self.title = None
            self.author = None
            self.language = None
            self.headline = None
            self.firstpublished = None
            self.genre = None
            self.group = None
            self.description = None
            self.series = None
            self.seriesnumber = None
            self.forgiveness = None
            self.starrating = None
            self.played = False
            self.coverart = None
        else:
            self.story_file = data[0]
            self.ifid = data[1]
            self.format = data[2]
            self.title = data[3]
            self.author = data[4]
            self.language = data[5]
            self.headline = data[6]
            self.firstpublished = data[7]
            self.genre = data[8]
            self.group = data[9]
            self.description = data[10]
            self.series = data[11]
            self.seriesnumber = data[12]
            self.forgiveness = data[13]
            self.starrating = data[14]
            self.played = data[15]
            self.coverart = data[16]

    def to_ifiction(self, xml_doc):
        story = xml_doc.createElement('story')
        identification = xml_doc.createElement('identification')
        story.appendChild(identification)
        ifidn = xml_doc.createElement('ifid')
        ifidn.appendChild(xml_doc.createTextNode(self.ifid))
        identification.appendChild(ifidn)
        formatn = xml_doc.createElement('format')
        formatn.appendChild(xml_doc.createTextNode(self.format))
        identification.appendChild(formatn)
        bibliographic = xml_doc.createElement('bibliographic')
        story.appendChild(bibliographic)

        if self.title != None:
            titlen = xml_doc.createElement('title')
            bibliographic.appendChild(titlen)
            titlen.appendChild(xml_doc.createTextNode(self.title))

        if self.author != None:
            authorn = xml_doc.createElement('author')
            bibliographic.appendChild(authorn)
            authorn.appendChild(xml_doc.createTextNode(self.author))

        if self.genre != None:
            genren = xml_doc.createElement('genre')
            bibliographic.appendChild(genren)
            genren.appendChild(xml_doc.createTextNode(self.genre))

        if self.description != None:
            desc = xml_doc.createElement('description')
            bibliographic.appendChild(desc)
            desc.appendChild(xml_doc.createTextNode(self.description))

        if self.language != None:
            languagen = xml_doc.createElement('language')
            bibliographic.appendChild(languagen)
            languagen.appendChild(xml_doc.createTextNode(self.language))

        if self.headline != None:
            headlinen = xml_doc.createElement('headline')
            bibliographic.appendChild(headlinen)
            headlinen.appendChild(xml_doc.createTextNode(self.headline))

        if self.firstpublished != None:
            publishedn = xml_doc.createElement('firstpublished')
            bibliographic.appendChild(publishedn)
            publishedn.appendChild(xml_doc.createTextNode(self.firstpublished))

        if self.group != None:
            groupn = xml_doc.createElement('group')
            bibliographic.appendChild(groupn)
            groupn.appendChild(xml_doc.createTextNode(self.group))

        if self.series != None:
            seriesn = xml_doc.createElement('series')
            bibliographic.appendChild(seriesn)
            seriesn.appendChild(xml_doc.createTextNode(self.series))

        if self.seriesnumber != None:
            seriesnumbern = xml_doc.createElement('seriesnumber')
            bibliographic.appendChild(seriesnumbern)
            seriesnumbern.appendChild(xml_doc.createTextNode(
                self.seriesnumber))

        if self.forgiveness != None:
            forgivenessn = xml_doc.createElement('forgiveness')
            bibliographic.appendChild(forgivenessn)
            forgivenessn.appendChild(xml_doc.createTextNode(self.forgiveness))

        if self.story_file != None:
            zoom = xml_doc.createElement('zoom')
            story.appendChild(zoom)
            storyfilen = xml_doc.createElement('story')
            storyfilen.appendChild(xml_doc.createTextNode(self.story_file))
            zoom.appendChild(storyfilen)
        return story

    def serialize(self):
        return [self.story_file, self.ifid, self.format, self.title,
                self.author, self.language, self.headline, self.firstpublished,
                self.genre, self.group, self.description, self.series,
                self.seriesnumber, self.forgiveness, self.starrating,
                self.played, self.coverart]
    
    def get_local_coverart(self):
        file_dir = os.path.dirname(self.story_file)
        local_coverart = glob(os.path.join(file_dir, '[Cc][Oo][Vv][Ee][Rr]*'))
        coverart = None
        for coverart in local_coverart:
            if os.path.splitext(coverart)[1] in ['.jpg', '.png', '.gif']:
                break
        self.coverart = coverart
